      PROGRAM ALS
*
*
*
*
* OBJECTIVE: TO SIMULATE PLANT WATER UPAKE AND SOIL WATER DYNAMICS
* 
* THE BACKBONE OF THIS PROGRAM IS A. W. WARRICK'S AUSTERE-LAYERED.
* SEE A. W. WARRICK, 2003. SOIL WATER DYNAMICS, OXFORD UNIVERSITY
* PRESS, 2003. NEW YORK.
* PROGRAM "AL.FOR" WAS EXTENDED AND USED IN GENERATING NUMERICAL
* SOLUTIONS FOR THE PAPER BY DONG ET AL. (2010) WHEN THE AUTHOR 
* WAS AFFILIATED WITH 
* NORTH DAKOTA STATE UNIVERSITY (FEBRUARY 26, 2010). 
*
*
* CURRETN AFFILIATION:
*
* XUEJUN DONG,
* TEXAS A&M AGRILIFE RESEARCH - UVALDE, TX
* EMAIL: XUEJUN.DONG@AG.TAMU.EDU 
*
*
*
* MAJOR MODIFICATIONS INCLUDE:
*
* (1) MODIFIED TO SIMULATE VARIABLY SATURATED SOIL, WITH
* FOUR STATEMENT FUNCTIONS (CONDF(), THETAF(), SWCF(), AND HEADF())
* REPLACED BY FOUR EXTERNAL SUBROUTINES,
* WITH FLUX UPDATING SCHEME ADAPTED FROM PROGRAM "A2&3.FOR", ALSO IN
* WARRICK (2003), WHICH WAS BASED ON KIRKLAND ET AL (1992).
*
* (2) ADDED DYNAMIC ROOT GROWTH AND ROOT UPTAKE AS SINK TERM
* INCLUDING ROOT UPTAKE FROM TOP 1 INCH SOIL AND DEEPER.
*
* (3) ADDED SURFACE SOIL EVAPORATION AND RAINFALL FLUXES 
*
* (4) ADDED THE EFFECT OF WATER STRESS ON ROOT WATER UPTAKE
*
* (5) MODIFIED TO INCLUDE THREE OPTIONS FOR 
* THE ENDING OPTIMAL SOIL WATER POTENTIAL (H3),
* AS WELL AS THREE TYPES OF ROOT WATER UPTAKE FUNCTIONS (OJHA'S
* COMPENSATION FUNCTION, LI'S COMPENSATION FUNCTION AND WU'S 
* FOUR-PARAMETER, NORMALIZED ROOT DISTRIBUTION FUNCTION). 
*
*
*
* INPUT FILES (SAME AS AL.FOR):
*
* (1) ALS2.DAT, INPUT SOIL PHYSICAL PARAMETERS & OTHER CONSTANTS.
* (2) WEA2.DAT, DAILY WEATHER DATA.  
* (3) INITF.DAT, INITIAL THETA PROFILE (INTERPOLATED).
* (4) TERMF.DAT, TERMINATING THETA PROFILE (MEASURED).
*
*
* OUTPUT FILES: 
*
* (1) OUTS.DAT, MIRRORS OUTPUT OF AL.F.
* (2) OUTS2.DAT, DAILY THETA/ H FOR INTERESTED DEPTHS.
* (3) INFS.DAT, MIRRORS INF.DAT OF AL.F.
* (4) WUSE.DAT, SUMMARY OF DAILY CUMMULATIVE WATER USE.
*
* SUBROUTINES PROVIDED:
*
* (1) CONDF (BASED ON A2&3.FOR).
* (2) HEADF (BASED ON A2&3.FOR).
* (3) SWCF (BASED ON A2&3.FOR). 
* (4) THETAF (BASED ON A2&3.FOR).
* (5) BORDER (BASED ON A2&3.FOR).  
* (6) BORD (BASED ON A2&3.FOR).		 	       
* (7) SINK
* (8) HTHREE  
* (9) THOMAS (SAME AS IN AL.FOR).
*
*
* SURFACE BOUNDARY CONDITIONS: 1-THETA/H    2-FLUX DENSITY
* FOR DIMENSIONS: USE CONSISTENT LENGTH ALONG WITH SECONDS FOR TIMES
*                 EXCEPT (1) FOR PRINTING WHERE HOURS ARE USED AND
*                 (2) FOR PRECIPITATION/EVAPOTRANSPIRATION, WHERE
*                 UNIT OF MM/DAY IS USED IN THE INPUT WEATHER FILE.

      PARAMETER (MAXLAY=12,NZMAX=500,NDMAX=300,JTHAW=105,JMATUR=196,
     1 BETA=1.5,CK=0.4,ZRMAX=1.2,IHTH=1,ICPSN=1)
      CHARACTER IDENTI*50
      INTEGER B_TYPE
      REAL KSAT(MAXLAY), M(MAXLAY), N(MAXLAY)
      DIMENSION A(NZMAX), ALPHA(MAXLAY), B(NZMAX), 
     1 C(NZMAX), COND(NZMAX), D(NZMAX), DEPF(MAXLAY), DEPTH(MAXLAY),
     1 DETP(NDMAX), DLAI(NDMAX), DRAIN(NDMAX), DRH(NDMAX),
     1 DTAVE(NDMAX), H(NZMAX), HSTAR(0:NZMAX), JULI(NDMAX),
     1 NL(0:NZMAX), SWC(NZMAX), THEF(MAXLAY), THETA(0:NZMAX),
     1 TIMPRT(50), T_R(MAXLAY),T_S(MAXLAY),TUPTAK(NZMAX),
     1 UPTAKE(NZMAX),Z(NZMAX)

*** INPUT
      OPEN(5, FILE='ALS2.DAT')
      READ(5,*) IDENTI
      READ(5,*) NZ,DZ
      READ(5,*) B_TYPE,BVALUE,INITIO,HINIT
      READ(5,*) DT,NINFP
      READ(5,*) NP
      DO I=1,NP
        READ (5,*) TIMPRT(I)
      END DO     
      READ(5,*) NLAYER
      DO I=1,NLAYER
        READ (5,*) DEPTH(I),T_R(I),T_S(I),ALPHA(I),M(I), KSAT(I)
        N(I)=1.0/(1.0-M(I))
      END DO     
      CLOSE (5)

      OPEN(5, FILE='WEA2.DAT', STATUS='OLD')
      READ(5,*) NW
      DO I=1,NW
        READ(5,*) JULI(I),DETP(I),DTAVE(I),DRAIN(I),DRH(I),DLAI(I)
      END DO
      CLOSE(5)

*** OPEN OUT FILES, 'OUTS' AND 'INFS.DAT', MIRROR INPUT
      OPEN(5, FILE='OUTS.DAT', STATUS='UNKNOWN')
      WRITE(5,102)
      WRITE(5,*) IDENTI
      WRITE(5,*)'AUSTERE-LAYERED FINITE DIFFERENCE, BY A.W. WARRICK'
      WRITE(5,*) 'MODIFIED BY X.J. DONG TO ADD ROOT WATER UPTAKE'
      WRITE(5,*) 'AND SOIL EVAPORATION FOR VARIABLY SATURATED SOIL'
      WRITE(5,102)
      WRITE(5,*) 'NZ,DZ, ', NZ, DZ
      WRITE(5,*) 'BOUNDARY TYPE,VALUE,INITYPE,HINIT', B_TYPE,BVALUE,
     1  INITIO,HINIT
      WRITE(5,*) 'DT AND FREQUENCY TO WRITE TO "INFS.DAT", ', DT,NINFP 
      WRITE(5,*)'NUMBER OF TIMES PRINT TO "OUTS.DAT", ',NP
      WRITE(5,*)'LAYER, LAYER LOWER DEPTH, THR, THS, ALPHA, M, KSAT:'
      DO I=1,NLAYER
        WRITE(5,100) I,DEPTH(I),T_R(I),T_S(I),ALPHA(I),M(I),KSAT(I)
      END DO       
      OPEN(6, FILE='INFS.DAT', STATUS='UNKNOWN')
      WRITE(6,*) 'T(H)    i/KSAT   I(LENGTH),    WBELOW(LENGTH)'

*** SETUP AND INITIALIZATION

      ILAYER=1
      WSTART=0.
      WBELOW=0.
      C_AEVA=0.
      C_ATRA=0.
      DO I=1,NZ
        TUPTAK(I)=0.        
      END DO

      IF (INITIO .EQ. 1) THEN
        OPEN(8, FILE='INITF.DAT')
        READ (8,*) NOBS
        IF (NOBS .NE. NZ) GO TO 999
        DO IZ=1,NZ
          READ(8,*) THETA(IZ)
        END DO
        CLOSE(8)

        DO IZ=1,NZ
          Z(IZ)=(IZ-0.5)*DZ
          IF (Z(IZ) .GT. DEPTH(ILAYER)) ILAYER=ILAYER+1
          NL(IZ)=ILAYER
	  CALL HEADF(THETA(IZ),ALPHA(ILAYER),M(ILAYER),N(ILAYER),
     1              T_R(ILAYER),T_S(ILAYER),HEADV)
          H(IZ)=HEADV
          WSTART=WSTART+DZ*THETA(IZ)
        END DO 

      ELSE 
        DO IZ=1,NZ
          Z(IZ)=(IZ-0.5)*DZ
          IF (Z(IZ) .GT. DEPTH(ILAYER)) ILAYER=ILAYER+1
          NL(IZ)=ILAYER
          H(IZ)=HINIT
          CALL THETAF(H(IZ),ALPHA(ILAYER),M(ILAYER),N(ILAYER),
     1              T_R(ILAYER),T_S(ILAYER),THETAV)
          THETA(IZ)=THETAV
          WSTART=WSTART+DZ*THETA(IZ)       
        END DO
      END IF    

      IF (B_TYPE .EQ. 1) THEN
        BIG_BV=(BVALUE-T_R(1))/(T_S(1)-T_R(1))
        CALL HEADF(BVALUE,ALPHA(1),M(1),N(1),T_R(1),T_S(1),HEADV)
	HZERO=HEADV
	CALL CONDF(BVALUE,ALPHA(1),KSAT(1),M(1),T_R(1),T_S(1),CONDV)
	CZERO=CONDV
        WRITE(5,*) 'H,K,AND BIG THETA AT Z=0:', HZERO,CZERO,BIG_BV
      END IF
      DZ2=DZ*DZ
      THET99=0.99*(T_S(1)-T_R(1))+T_R(1)
      CALL CONDF(THET99,ALPHA(1),KSAT(1),M(1),T_R(1),T_S(1),CONDV)
      CALL SWCF(THET99,ALPHA(1),M(1),N(1),T_R(1),T_S(1),SWCV)
      DIFF99=CONDV/SWCV
      FOURIE=DIFF99*DT/DZ2
      WRITE(5,*) 'FOURIER GRID NUMBER AT SURFACE FOR BIGTH=0.99',
     1            ' IS', FOURIE
      IPR=1
      IT=1
      BIGI=0.
      WBELOW=0.
*** flux=bvalue for type 2 (will alter value later for type 1)
      FLUX=BVALUE  
      T=0.
      ND=1

      OPEN(8, FILE='WUSE.DAT', STATUS='UNKNOWN')
      WRITE(8,*) 'JULIAN, CUM EVAP(M), CUM TRAN(M)'

***  OPEN THE FILE CONTAINING TERMINAITNG WATER CONTENTS 
      OPEN(10, FILE='TERMF.DAT', STATUS = 'OLD')
      READ (10,*) NN
      DO I=1,NN
        READ (10,*) DEPF(I),THEF(I)
      END DO     
      CLOSE(10)

*** OPEN A FILE TO STORE THETA AND H AT END OF EACH DAY 
*** FOR INTERESTED DEPTHS
      OPEN(10, FILE='OUTS2.DAT', STATUS='UNKNOWN')
      WRITE(10,*) 'JULIAN  DEPTH(M)  THETA    H(M)'        

*** START TIME ITERATION
      PRINT *, 'TIME AT END OF EACH 100 ITERATIONS FOLLOW:'
      DO WHILE (T/3600 .LT. TIMPRT(NP))
        IT=IT+1
	T=T+DT
	IF (MOD(IT,100) .EQ. 0) PRINT *, 'T(H)  ', T/3600

*** UPDATE NUMBER OF DAYS SINCE THE START OF SIMULATION AND
*** PRINT CUMMULATIVE DAILY WATER USE AT THE END OF EACH DAY.

      IF (REAL(IT) .GT. 86400./DT*ND) THEN
	WRITE(8,105) JULI(ND), C_AEVA, C_ATRA
        DO I=1,NN
	  DO J=1,NZ
	    IF ((DEPF(I) .GT. Z(J)-0.5*DZ) .AND. (DEPF(I) .LT.
     1        Z(J)+0.5*DZ)) THEN
              WRITE(10,107) JULI(ND),DEPF(I),THETA(J),H(J)
            END IF
	  END DO
        END DO
        ND=ND+1   
      END IF

*** COMPUTE TPD AND ZRT	
      JULID=JULI(ND)
      ETPD=DETP(ND)
      DL=DLAI(ND)
      TAVE=DTAVE(ND)
      RAIN=DRAIN(ND)
      RH=DRH(ND)
      EP=ETPD*EXP(-CK*DL)
      TPD=ETPD-EP
      DNORM=(REAL(JULID)-REAL(JTHAW))/(REAL(JMATUR)-REAL(JTHAW))
      IF ((JULID .GT. JTHAW) .AND. (JULID .LE. JMATUR)) THEN
	ZRT=ZRMAX*(6.372/(5.372*(1.0+5.372*EXP(-6.105*DNORM)))
     1  -1.0/5.372)
      ELSE IF (JULID .GT. JMATUR) THEN
	ZRT=ZRMAX
      ELSE
	ZRT=0.025	
      END IF    

*** COMPUTE S(Z,T) WITH UNIT OF 1/DAY
      CALL SINK(H,Z,NZ,DZ,ZRT,BETA,TPD,IHTH,ICPSN,UPTAKE)

*** CHANGE UNIT OF S(Z,T) TO 1/S AND UPDATE CUMMUL TRANSPIRATION, IN M
      DO IZ=1,NZ
        UPTAKE(IZ)=UPTAKE(IZ)/86400.
        UPDTDZ=UPTAKE(IZ)*DT*DZ   
        TUPTAK(IZ)=TUPTAK(IZ)+UPDTDZ   
        C_ATRA=C_ATRA+UPDTDZ 
      END DO 

*** SET UP COEFFICIENTS (NOTE COND(I) IS FOR K AT I-0.5)	  
      C2=0
      IF (B_TYPE .EQ. 1) C2=CZERO
      DO IZ=1,NZ
        C1=C2
        CALL CONDF(THETA(NL(IZ)),ALPHA(NL(IZ)),KSAT(NL(IZ)),
     1  	    M(NL(IZ)),T_R(NL(IZ)),T_S(NL(IZ)),CONDV)
      C2=CONDV
      COND(IZ)=0.5*(C1+C2)
      CALL SWCF(THETA(NL(IZ)),ALPHA(NL(IZ)),M(NL(IZ)),N(NL(IZ)),
     1             T_R(NL(IZ)),T_S(NL(IZ)),SWCV)
      SWC(IZ)=SWCV     
      END DO	
      COND(NZ+1)=C2

      DO IZ=1,NZ
        A(IZ)=-COND(IZ)/DZ2
	B(IZ)=SWC(IZ)/DT+(COND(IZ+1)+COND(IZ))/DZ2
	C(IZ)=-COND(IZ+1)/DZ2
	D(IZ)=SWC(IZ)/DT*H(IZ)-(COND(IZ+1)-COND(IZ))/DZ-UPTAKE(IZ)
      END DO
      
      IF (B_TYPE .EQ. 1) THEN
        D(1)=D(1)+COND(1)*HZERO/DZ2-UPTAKE(1)
      ELSE
        B(1)=SWC(1)/DT+COND(2)/DZ2
	D(1)=SWC(1)/DT*H(1)+(FLUX-COND(2))/DZ-UPTAKE(1)
      END IF
      
      B(NZ)=SWC(NZ)/DT+COND(NZ)/DZ2
      D(NZ)=SWC(NZ)/DT*H(NZ)-(COND(NZ+1)-COND(NZ))/DZ-UPTAKE(NZ)

*** COMPUTE SURFACE FLUX AT NEW TIME
      RAIN=RAIN/1000./86400.
      EP=EP/1000./86400.
      PSI=8.31*(TAVE+273.16)*LOG(RH)/0.018/9.81
      EAT=COND(1)*(H(1)-PSI)*2.0/DZ
      IF (EAT .GE. EP) EAT=EP
      FLUX=RAIN-EAT

*** UPDATE CUMMULATIVE SOIL EVAPORATION (ACTUAL), IN M
      C_AEVA=C_AEVA+EAT*DT 

*** UPDATE COMMULATIVE & INSTANT INFILTRATION, IN M
      IF (RAIN .LT. KSAT(1)) THEN
        BIGI=BIGI+RAIN*DT
        SMALLI=RAIN/KSAT(1)
      ELSE  
        BIGI=BIGI+KSAT(1)*DT
        SMALLI=1.0
      END IF

      WBELOW=WBELOW+COND(NZ)*DT
      IF (MOD(IT,NINFP) .EQ. 0) WRITE(6,104) T/3600,SMALLI,BIGI,WBELOW

*** GET SOLUTIONS AND WRITE TO "INFS.DAT", AND DO FLUX UPDATING
      CALL THOMAS(NZ,A,B,C,D,HSTAR)

      CALL BORDER(NZ,NZMAX,DZ,HSTAR)
      F2=FLUX
      IF (B_TYPE .EQ. 1) F2=COND(1)*(1.0-(HSTAR(1)-HZERO)/DZ)
      
      DO IZ=1,NZ
        F1=F2
        F2=COND(IZ+1)*(1.0-(HSTAR(IZ+1)-HSTAR(IZ))/DZ)
        THETA(IZ)=THETA(IZ)-DT/DZ*(F2-F1)-DT*UPTAKE(IZ)
      END DO

      CALL BORDER(NZ,NZMAX,0.0,THETA)
      CALL BORD(NZ,NZMAX,NL)
      DO IZ=1,NZ
        IF ((HSTAR(IZ) .GE. 0.) .OR. (HSTAR(IZ-1) .GE. 0.) .OR.
     1      (HSTAR(IZ+1) .GE. 0.) .OR. (THETA(IZ) .GE. T_S(NL(IZ))) 
     1      .OR. (THETA(IZ-1) .GE. T_S(NL(IZ-1))) .OR.
     1      (THETA(IZ+1) .GE. T_S(NL(IZ+1)))) THEN
          H(IZ)=HSTAR(IZ)
          CALL THETAF(HSTAR(IZ),ALPHA(NL(IZ)),M(NL(IZ)),
     1  	    N(NL(IZ)),T_R(NL(IZ)),T_S(NL(IZ)),THETAV)
          THETA(IZ)=THETAV
        ELSE
          CALL HEADF(THETA(IZ),ALPHA(NL(IZ)),M(NL(IZ)),
     1  	    N(NL(IZ)),T_R(NL(IZ)),T_S(NL(IZ)),HEADV)
          H(IZ)=HEADV
        END IF
      END DO

*** CHECK WHETHER TO PRINT AND DO IT IF READY
      IF (T/3600-TIMPRT(IPR) .GE. 0.) THEN
        WATER=0.
        DO IZ=1,NZ
	  WATER=WATER+DZ*THETA(IZ)
        END DO
        WADD=WATER-WSTART+WBELOW+C_AEVA+C_ATRA
        WRITE(5,102)
        WRITE(5,*)'     T(H)    T(MIN)    i/KSAT(1)   I(M)',
     1    '    WADD(M)   % DIFF'
        WRITE(5,103)T/3600,T/60,SMALLI,BIGI,WADD,(BIGI-WADD)/WADD*100
        WRITE(5,102)
        WRITE(5,*) ' Z       H        THETA    TUPTAKE'
        DO IZ=1,NZ
          WRITE (5,101) Z(IZ), H(IZ), THETA(IZ), TUPTAK(IZ)
        END DO
        IPR=IPR+1
      END IF
		
*** END OF WHILE
      END DO	

*** COMPUTE THE AVERAGE RELATIVE DISCREPANCY (ARD) BETWEEN MEASURED
*** AND PREDICTED SOIL WATER CONTENTS AT THE END OF TERMINATING DAY 
      ARD=0.
      DO I=1,NN
	DO J=1,NZ
	  IF ((DEPF(I) .GT. Z(J)-0.5*DZ) .AND. (DEPF(I) .LT.
     1  Z(J)+0.5*DZ)) THEN
          ARD=ARD+ABS(THETA(J)-THEF(I))
          END IF
	END DO
      END DO

      THEFM=0.
      DO I=1,NN
        THEFM=THEFM+THEF(I)
      END DO
      THEFM=THEFM/NN
      
      ARD=ARD/THEFM/NN
      WRITE(8,*) 'JULIAN DAY   AVERAGE RELATIVE DISCREPANCY IN THETA'
      WRITE(8,106) JULI(ND), ARD 
      PRINT *, 'IT IS DONE. SEE FILES: OUTS,INFS.DAT, AND WUSE.DAT '

 100  FORMAT(I1, 1F5.2, 4F6.3, 1E10.3)     
 101  FORMAT(F7.3, E11.3, 1F7.3, E11.3)
 102  FORMAT(' ', 75('-'))
 103  FORMAT(F10.2, 3E10.2, 2F10.3)
 104  FORMAT(2F10.3, 2E10.3)   
 105  FORMAT(I10, 2F16.7)                  
 106  FORMAT(I10, F16.4)  
 107  FORMAT(I4,2F6.3,E11.3)
      CLOSE(5)
      CLOSE(6)
      CLOSE(8)
      CLOSE(10)
 999  PRINT *, '(NUMBER OF POINTS) .NE. NZ! CHECK FILE "INITF.DAT"! '
      END


*---- RETURNS REAL CONDUCTIVITY AS CONDV BASED ON REAL THETA (TH)
      SUBROUTINE CONDF(TH,ALPHA,RKSAT,RM,THR,THS,CONDV)
      
      IF (TH .GE. THS) THEN
        CONDV=RKSAT
      ELSE
        XX=(TH-THR)/(THS-THR)
        CONDV=RKSAT*(XX**0.5)*(1.-(1.-XX**(1./RM))**RM)**2
      END IF  
      RETURN
      END

*---- RETURNS REAL HEAD AS HEADV BASED ON REAL THETA (TH)
      SUBROUTINE HEADF(TH,ALPHA,RM,RN,THR,THS,HEADV)
      
      IF (TH .GE. THS) THEN
        HEADV=0.
      ELSE
        XX=(TH-THR)/(THS-THR)
        HEADV=-(XX**(-1./RM)-1.)**(1./RN)/ALPHA
      END IF  
      RETURN
      END

*---- RETURNS REAL SOIL WATER CAPACITY SWCV BASED ON REAL THETA (TH)
      SUBROUTINE SWCF(TH,ALPHA,RM,RN,THR,THS,SWCV)
      
      IF (TH .GE. THS) THEN
        SWCV=0.
      ELSE
        XX=(TH-THR)/(THS-THR)
        SWCV=ALPHA*(THS-THR)*RM*RN*XX**(1./RM+1.)*(XX**(-1./RM)-1.)
     1     **(1.-1./RN)
      END IF  
      RETURN
      END

*---- RETURNS REAL THETA AS THETAV BASED ON REAL HEAD (HH)
      SUBROUTINE THETAF(HH,ALPHA,RM,RN,THR,THS,THETAV)
      
      IF (HH .GE. 0.) THEN
        THETAV=THS
      ELSE
        THETAV=THR+(THS-THR)*(1+ABS(ALPHA*HH)**RN)**(-RM)
      END IF  
      RETURN
      END

*---- SETS VALUES FOR IMAGINARY NODES EQUAL TO ADJACENT NODES.
*     USE FOR BOTH H() AND THETA(). FOR IZ=0 DECREASES BY ADDITIONAL
*     ZFACTOR USEFUL FOR DECREASE DUE TO GRAVITY FOR H.
*     USE ZFACTOR=0.0 FOR THETA().
*     FOR TESTING IF A NODE, OR ITS IMMEDIATELY
*     ADJACENT NODES, IS NEAR SATURATION (FOR POSSIBLE FLUX UPDATING). 
      SUBROUTINE BORDER(NZ,NZMAX,ZFACT,HH)

      REAL HH(0:NZMAX)
      DO IZ=0,NZ+1
        IF (IZ .EQ. O) THEN
          HH(IZ)=HH(1)-ZFACT
        ELSE IF (IZ .EQ. NZ+1) THEN
          HH(IZ)=HH(NZ)
        ELSE
          HH(IZ)=HH(IZ)+0.
        END IF
      END DO
      RETURN
      END

*---- SETS VALUES FOR IMAGINARY NODES EQUAL TO ADJACENT NODES.
*     USE ONLY FOR NL() FOR TESTING IF A NODE, OR ITS IMMEDIATELY
*     ADJACENT NODES, IS NEAR SATURATION (FOR POSSIBLE FLUX UPDATING).  
      SUBROUTINE BORD(NZ,NZMAX,NLL)

      DIMENSION NLL(0:NZMAX)

      DO IZ=0,NZ+1
        IF (IZ .EQ. O) THEN
          NLL(IZ)=NLL(1)
        ELSE IF (IZ .EQ. NZ+1) THEN
          NLL(IZ)=NLL(NZ)
        ELSE
          NLL(IZ)=NLL(IZ)+0.
        END IF
      END DO
      RETURN
      END
 
*---- COMPUTE THE ROOT UPTAKE TERM
      SUBROUTINE SINK(H,Z,NZ,DZ,ZRT,B,TPD,IH,ICP,S)
      
      PARAMETER(NMAX=500)
      DIMENSION H(NMAX),Z(NMAX),GAMMA(NMAX),DLNRD(NMAX),S(NMAX)

*---- IH   1: H=(-11,-5);
*          2: H=(-14,-8);
*          3: H=(-16,-10)
*---- ICPS 1: OJHA'S COMPENSATION FUNCTION
*          2: LI'S COMPENSATION FUNCTION
*          3. WU'S FUNCTION (WITHOUT COMPENSATION)            
      H1=-0.1
      H2=-0.25
      H4=-152.85
      TPDM=TPD/1000.

      CALL HTHREE(IH,TPD,H3)

      DO IZ=1,NZ
        IF ((H(IZ) .GE. H4) .AND. (H(IZ) .LT. H3)) THEN
          GAMMA(IZ)=(H(IZ)-H4)/(H3-H4)
        ELSE IF ((H(IZ) .GE. H3) .AND. (H(IZ) .LT. H2)) THEN
          GAMMA(IZ)=1.0
        ELSE IF ((H(IZ) .LT. H4) .OR. (H(IZ) .GE. H1)) THEN
          GAMMA(IZ)=0.
        ELSE
          GAMMA(IZ)=(H(IZ)-H1)/(H2-H1)          
        END IF
      END DO       

      IF (ICPS .EQ. 1) THEN
        DO IZ=1,NZ
          GZT=(B+1.0)/ZRT*(1.0-Z(IZ)/ZRT)**B
          IF (Z(IZ) .LE. ZRT) THEN
            S(IZ)=GAMMA(IZ)*GZT*TPDM
          ELSE
            S(IZ)=0.
          END IF
        END DO
      ELSE IF (ICPS .EQ. 2) THEN
        DO IZ=1,NZ
          IF (Z(IZ) .LE. ZRT) THEN
            DLNRD(IZ)=2.21-3.72*Z(IZ)/ZRT+3.46*(Z(IZ)/ZRT)**2
     1             -1.87*(Z(IZ)/ZRT)**3
          ELSE
            DLNRD(IZ)=1.
          END IF
        END DO

        DENOM=0.
        DO IZ=1,NZ
          IF (Z(IZ) .LE. ZRT) THEN
            DENOM=DENOM+GAMMA(IZ)*DLNRD(IZ)**B
          ELSE
            DENOM=DENOM
          END IF
        END DO
 
        DO IZ=1,NZ        
          IF (Z(IZ) .LE. ZRT) THEN
            S(IZ)=GAMMA(IZ)**2*DLNRD(IZ)**B*TPDM/DZ/DENOM
          ELSE
            S(IZ)=0.
          END IF
        END DO
      ELSE
        DO IZ=1,NZ
          IF (Z(IZ) .LE. ZRT) THEN
            DLNRD(IZ)=2.21-3.72*Z(IZ)/ZRT+3.46*(Z(IZ)/ZRT)**2
     1             -1.87*(Z(IZ)/ZRT)**3
          ELSE 
            DLNRD(IZ)=1.
          END IF            
        END DO        
    
        DO IZ=1,NZ
          IF (Z(IZ) .LE. ZRT) THEN
            S(IZ)=GAMMA(IZ)*DLNRD(IZ)*TPDM/ZRT
          ELSE
            S(IZ)=0.
          END IF
        END DO
      END IF
      RETURN
      END

*---- RETURNS H3 FOR SINK TO COMPUTE ROOT UPTAKE
*     'IIH' IS AN INTEGER (1, 2, OR 3) INDICATING DIFFERENT
*     OPTIONS FOR H3
*     'T' IS DAILY POTENTIAL TRANSPIRATION (TPD)
*     'HT' IS THE RETURNED VALUE OF H3
      SUBROUTINE HTHREE(IIH,T,HT)

      TP1=1.0
      TP2=5.0
      IF (IIH .EQ. 1) THEN
        H31=-11.0
        H32=-5.0
      ELSE IF (IIH .EQ. 2) THEN
        H31=-14.0
        H32=-8.0
      ELSE
        H31=-16.0
        H32=-10.0
      END IF

      A=(H31-H32)/(TP1-TP2)
      IF ((T .GE. 1.0) .AND. (T .LE. 5.0)) THEN
	HT=A*T+H31-A*TP1
      ELSE IF (T .LT. 1.0) THEN
        HT=H31
      ELSE
        HT=H32
      END IF
      RETURN
      END

*---- THOSMAS FOR TRIDIAGONAL SOLUTION
      SUBROUTINE THOMAS(N,A,B,C,D,X)

      PARAMETER (NMAX=500)
      DIMENSION A(NMAX),B(NMAX),C(NMAX),D(NMAX), X(0:NMAX),
     1  ALPHA(NMAX),BETA(NMAX),Y(NMAX)

      ALPHA(1)=B(1)
      BETA(1)=C(1)/ALPHA(1)
      Y(1)=D(1)/ALPHA(1)
      DO I=2,N
        ALPHA(I)=B(I)-A(I)*BETA(I-1)
        BETA(I)=C(I)/ALPHA(I)
        Y(I)=(D(I)-A(I)*Y(I-1))/ALPHA(I)
      END DO

      X(N)=Y(N)
      DO I=1,N-1
        J=N-I
        X(J)=Y(J)-BETA(J)*X(J+1)
      END DO
      RETURN
      END
